/**
 * Character group.
 * @author Andre Platzer
 * @version 2003-08-23
 */
:- module char_group.
:- interface.
:- import_module char, basecap.
:- use_module set.

/**
 * A character group.
 */
:- type char_group.
%%@xxx should abstract this
:- type char_group == set__set(char).
%%:- instance printable(char_group).

:- func make_single_group(char) = char_group.
:- func make_range_group(char,char) = char_group.
:- func union(char_group,char_group) = char_group.
:- func intersect(char_group,char_group) = char_group.
:- func difference(char_group,char_group) = char_group.

/**
 * Get the char_group where a character belongs to.
 * group(Gs, C)
 * Fails if the given set Gs of char_groups that characters can belong to
 * does not contain C somewhere.
 * Errors if the given set Gs is non-disjoint, i.e. some characters would fall into
 * more than one char_group.
 */
:- func group(set__set(char_group), char) = char_group.
:- mode group(in, in) = out is semidet.


:- implementation.
:- import_module set, list, string.

%%:- type char_group == set(char).
make_single_group(C) = make_singleton_set(C).
make_range_group(C1,C2) = list_to_set(map(semidet_to_det(
							 (func(C::in) = (D::out) is semidet
							 :- to_int(D,C)
							 ),
							 "character conversion for ranges"),
							 to_int(C1) `..` to_int(C2))).
union(C1,C2) = set__union(C1,C2).
difference(C1,C2) = set__difference(C1,C2).
intersect(C1,C2) = set__intersect(C1,C2).


/*
:- instance printable(char_group) where [
	to_string(CG) = "char_group[" ++ join_list(", ", to_sorted_list(map(to_string, CG))) ++ "]"
 ].
*/

group(Gs, C) = some_solution((pred(G::out) is nondet :-
			      member(G, Gs),
			      member(C, G)
			     )).
%% @todo error in case of overlaps

:- end_module char_group.