/**
 * Provides tools for abstracting individual character tokens to tokens
 * of the same value, but only a character range as kind.
 * @author Andre Platzer
 * @version 2003-08-25
 */
:- module char_grouper.
:- interface.

:- import_module scanners, char_group, set.


:- all [TokenSequence] some [SkippingTokenSequence] (
	func char_group_filter(set(char_group),TokenSequence) = SkippingTokenSequence
		=> tokenSequence(SkippingTokenSequence)
	) <= tokenSequence(TokenSequence).

/*
:- all [TokenSequence] some [SkippingTokenSequence] (
	func skip_filter(TokenSequence) = SkippingTokenSequence
		=> tokenSequence(SkippingTokenSequence)
	) <= tokenSequence(TokenSequence).
*/


:- implementation.

:- import_module std_util, string_source, basecap, require, string.

char_group_filter(Groups,TS) = map_univ(
					(func(Tok `with_type` char_token) = GroupTok :-
					%%log_print("\tfiltering"++to_string(Tok)),
					(char_token(C) = cast(Tok) ->
					    log_print("\t\tgroup character " ++ to_string(C)),
					    GroupTok = make_token_kind(to_string(group(Groups,C)))
					;
					    end_of_file_source = cast(Tok),
					    GroupTok = end_of_file
					)
					;
					    report_lookup_error("assignment to char_group failed", Tok, GroupTok)
					), TS).

/*
skip_filter(TS) = filter_map_univ(
				  (func(Tok::in) = (Tok::out) is semidet :-
				  %%log_print("\tfiltering"++to_string(Tok)),
				  univ_to_type(Tok, Tok2),
				  get_kind(Tok2) \= scanners__skip), TS).
*/
:- end_module char_grouper.