/**
 * Provides a tokenizer that regards each single character as a token.
 * @author Andre Platzer
 * @version 2003-08-20
 */
:- module char_tokenizer.
:- interface.
:- import_module scanners, char, basecap.

/**
 * Create a char tokenizer separating each single character,skipping whitespaces.
 */
:- some [S] (func char_tokenizer = S => scanner(S)).
:- mode char_tokenizer = out is det.

:- type char_token
	---> character(
		  signifier :: char
		 )
	;    endoffile.
:- instance printable(char_token).
:- instance token(char_token).


:- implementation.

:- import_module list, string, char, require, std_util, basecap.
:- import_module scanner_util.


:- some[TS] func scan(string) = TS => tokenSequence(TS).
:- mode scan(in) = out is semidet.


:- type charTokenizer
	---> charTokenizer.

:- instance printable(charTokenizer) where [
	to_string(_) = "charTokenizer tokenizes individual characters, skipping whitespaces"
  ].
:- instance scanner(charTokenizer) where [
	scan(S, Input) = scan(Input)
  ].

char_tokenizer = charTokenizer.


:- type token == char_token.

:- instance printable(char_token) where [
        to_string(character(A)) = char_to_string(A),
        to_string(endoffile) = "eof"
].

:- instance token(char_token) where [
	(get_value(character(A)) = char_to_string(A)),
	(get_value(endoffile) = A :- error("cannot get_value of " ++ to_string(endoffile))),

        get_kind(character(A)) = make_token_kind(char_to_string(A)),
        get_kind(endoffile) = make_token_kind("end_of_file"),

	end_of_file = endoffile
 ].


scan(S) = list__reverse(Tokens) :-
	scan(to_char_list(S), [], Tokens).
:- pred scan(list(char), list(token), list(token)).
:- mode scan(in, in, out) is det.
scan([], Toks, [endoffile | Toks]).
scan([C | Cs], Toks0, Toks) :-
    (
        char__is_whitespace(C)
    ->
        scan(Cs, Toks0, Toks)
    ;
        scan(Cs, [character(C) | Toks0], Toks)
    ).

:- end_module char_tokenizer.