:- module np.
:- interface.
:- import_module io.

:- pred main(io__state::di, io__state::uo) is det.

:- implementation.

:- import_module list, set, string, basecap, scanners, parsers, std_util, require,
	parsers__syntheticParser, parsers__sllParser,
	parsers__analyticParser, parsers__lr1Parser,
	parser_generator, parser_generator__lr1Generator, parser_generator__sllGenerator,
	grammar_analysis,
	syntaxTree.
:- use_module map, assoc_list.
:- import_module simple_tokenizer, token_impl.

:- pred go(string::in, actionType :: out, string::out) is nondet.
go(I, O, Logbook) :-
	G0 = grammar(nonterminal("S"),
		     [
		      rule(nonterminal("S"), [nonterminal("S"), 'new terminal'(identifier("x"))]),
		      rule(nonterminal("S"), ['new terminal'(endoffile)])
		     ]
		    ),
	PT0 =
	[
	 {['new terminal'(endoffile)], nonterminal("S"), []},
	 {['new terminal'(identifier("x"))], nonterminal("S"), [nonterminal("S"), 'new terminal'(identifier("x"))]}
	]
	,
	%% Proper Term grammar
	G1 = grammar(nonterminal("S"),
		     [
		      rule(nonterminal("S"), [nonterminal("expr")]),
		      rule(nonterminal("expr"), [nonterminal("expr"), 'new terminal'(operator("+")), nonterminal("term")]
			  /*,
			   (func(Vals) = R :-
					  Vals=[V1,V2,V3] ->
					  R = new_action_value("("++to_string(V1) ++ "+" ++ to_string(V3)++")")
					 ;
					  error("NOGO1"))
			   */),
		      rule(nonterminal("expr"), [nonterminal("term")]),
		      rule(nonterminal("term"), [nonterminal("term"), 'new terminal'(operator("*")), nonterminal("factor")]
			  /*,
			   (func(Vals) = R :-
					  Vals=[V1,V2,V3] ->
					  R = new_action_value("("++to_string(V1) ++ "*" ++ to_string(V3)++")")
					 ;
					  error("NOGO2"))
			   */),
		      rule(nonterminal("term"), [nonterminal("factor")]),
		      rule(nonterminal("factor"), ['new terminal'(operator("(")), nonterminal("expr"), 'new terminal'(operator(")"))]),
		      rule(nonterminal("factor"), ['new terminal'(identifier("x"))])
		     ]
		    ),
	%% Factorized Term grammar
	G2 = grammar(nonterminal("S"),
		     [
		      rule(nonterminal("S"), [nonterminal("expr")]),
		      rule(nonterminal("expr"), [nonterminal("term"), nonterminal("expr1")]),
		      rule(nonterminal("expr1"), []),
		      rule(nonterminal("expr1"), ['new terminal'(operator("+")), nonterminal("expr")]),
		      rule(nonterminal("term"), [nonterminal("factor"), nonterminal("term1")]),
		      rule(nonterminal("term1"), []),
		      rule(nonterminal("term1"), ['new terminal'(operator("*")), nonterminal("term")]),
		      rule(nonterminal("factor"), ['new terminal'(identifier("x"))]),
		      rule(nonterminal("factor"), ['new terminal'(operator("(")), nonterminal("expr"), 'new terminal'(operator(")"))])
		     ]
		    ),
	PTD2 =
	[
	 {['new terminal'(identifier("x"))], nonterminal("S"), [nonterminal("expr")]},
	 {['new terminal'(operator("("))], nonterminal("S"), [nonterminal("expr")]},
	 {['new terminal'(identifier("x"))], nonterminal("expr"), [nonterminal("term"), nonterminal("expr1")]},
	 {['new terminal'(operator("("))], nonterminal("expr"), [nonterminal("term"), nonterminal("expr1")]},
	 {['new terminal'(operator(")"))], nonterminal("expr1"), []},
	 {['new terminal'(endoffile)], nonterminal("expr1"), []},
	 {['new terminal'(operator("+"))], nonterminal("expr1"), ['new terminal'(operator("+")), nonterminal("expr")]},
	 {['new terminal'(identifier("x"))], nonterminal("term"), [nonterminal("factor"), nonterminal("term1")]},
	 {['new terminal'(operator("("))], nonterminal("term"), [nonterminal("factor"), nonterminal("term1")]},
	 {['new terminal'(operator("+"))], nonterminal("term1"), []},
	 {['new terminal'(operator(")"))], nonterminal("term1"), []},
	 {['new terminal'(endoffile)], nonterminal("term1"), []},
	 {['new terminal'(operator("*"))], nonterminal("term1"), ['new terminal'(operator("*")), nonterminal("term")]},
	 {['new terminal'(identifier("x"))], nonterminal("factor"), ['new terminal'(identifier("x"))]},
	 {['new terminal'(operator("("))], nonterminal("factor"), ['new terminal'(operator("(")), nonterminal("expr"), 'new terminal'(operator(")"))]}
	],
	PT2 =
	   (func(Tokens,Symbol) = R
	   :- [R|_] = solutions((pred(X::out) is nondet :- member({Tokens,Symbol,X}, PTD2)))
	   ; error("did not find " ++ to_string(Symbol) ++ " with LA=" ++ to_string(Tokens) ++ " in parseTable")
	   )
	   `with_type` (func(list(Token),symbol)=list(symbol))
	,
	G3 = grammar(nonterminal("E"),
		     [
		      rule(nonterminal("E"), ['new terminal'(identifier("x")), nonterminal("E")]
			  %%,(func(Vals) = new_action_value("("++to_string(index0_det(Vals,0)) ++ "~" ++ to_string(index0_det(Vals,1))++")"))
		      ),
		      rule(nonterminal("E"), ['new terminal'(identifier("x"))])
		     ]),
	ATD3 = map__from_assoc_list(assoc_list__from_corresponding_lists(
		[
		 {0,'new terminal'(identifier("x"))},
		 {1,'new terminal'(identifier("x"))},
		 {1,'new terminal'(endoffile)},
		 {2,'new terminal'(endoffile)},
		 {3,'new terminal'(endoffile)}
		],
		[
		 shift,
		 shift,
		 reduce(rule(nonterminal("E"), ['new terminal'(identifier("x"))])),
		 accept,
		 reduce(index0_det(productions(G3),0)) %%reduce(rule(nonterminal("E"), ['new terminal'(identifier("x")), nonterminal("E")]))
		])),
	%%AT3 = ((func(State,Symbol) = map__lookup(ATD3, {State,Symbol}))
	%%      `with_type` (func(int,Token)=parserAction)),
	AT3 = ((func(State,Symbol) = R
	       :- R = map__search(ATD3, {State,Symbol})
	       ; error("did not find q=" ++ to_string(State) ++ " a=" ++ to_string(Symbol) ++ " in actionTable")
	       )
	      `with_type` (func(int,Token)=parserAction)),
	GTD3 = map__from_assoc_list(assoc_list__from_corresponding_lists(
		[
		 {0,'new terminal'(identifier("x"))},
		 {1,'new terminal'(identifier("x"))},
		 {0,nonterminal("E")},
		 {1,nonterminal("E")}
		],
		[
		 1,
		 1,
		 2,
		 3
		])),
	GT3 = ((func(State,Symbol) = R
	       :- R = map__search(GTD3, {State,Symbol})
	       ; error("did not find q=" ++ to_string(State) ++ " a=" ++ to_string(Symbol) ++ " in gotoTable")
	       )
	      `with_type` (func(int,Token)=int)),
	G5 = grammar(nonterminal("S"),
		     [
		      rule(nonterminal("S"), ['new terminal'(identifier("a")), nonterminal("A"), 'new terminal'(identifier("c"))]),
		      rule(nonterminal("A"), [nonterminal("A"), 'new terminal'(identifier("b")), 'new terminal'(identifier("b"))]),
		      rule(nonterminal("A"), ['new terminal'(identifier("b"))])
		     ]),
	TS = scan(simple_tokenizer,I),
	%%synthetic_parser(G2) = P,
	%%sll_parser(G2, PT2) = P,
	%%analytic_parser(G1) = P,
	%%lr1_parser(G3,0,AT3,GT3) = P,

	lr1_parser(G1) = P,
	%%sll_parser(G2) = P,
	Logbook = to_string(P),
	log_print(Logbook),
	parse(P, TS, O).



main -->
	write_string("Input line: "),
	read_line_as_string(Result),
	{ Result = ok(I)
	; error("NO")
	}
	,
	(
	  {Toks = scan(simple_tokenizer,I)} ->
	  write_string("Scanned "),
	  write(Toks)
	;
	  write_string("lexical error")
	),
	write_string("\n\n"),
	{
	 solutions((pred((to_string(X) ++ "\n\nLog:\n" ++ Str)::out) is nondet :- go(I,X,Str)), O)
	},
	write_string("Productions\n"),
	write_list(O, "\n\n",  print ),
	write_string("\n").
