/**
 * Scat, the scanner generator test module.
 * @author Andre Platzer
 * @version 2003-08-20
 */
:- module scat.
:- interface.
:- import_module io.

:- pred main(io__state::di, io__state::uo) is det.

:- implementation.

:- import_module list, set, string, basecap, scanners, std_util, require,
	scanner_generator, scanner_generator__lr0Generator,
	scanners__dfa.
:- import_module simple_tokenizer, token_impl, scanner_util.

main -->
	write_string("Welcome to Scat!\n"),
	{
	 LexicalGrammar = grammar(
		     [
		      rule(make_token_kind("id_a"), ['a']),
		      rule(make_token_kind("digit"), ['0']),
		      rule(make_token_kind("sid"), to_char_list("(b|1)x"))
		     /*
		      rule(make_token_kind("id_a"), ['a']),
		      rule(make_token_kind("id_b"), ['b']),
		      rule(make_token_kind("digit"), ['0']),
		      rule(make_token_kind("number"), ['1', '2']),
		      skip(['\r']),
		      skip(['\n'])
		       */
		     ]
		    ),
	  Scanner = dfa_scanner(LexicalGrammar)
	;
	  error("Lexical grammar cannot be parsed")
	},
	{ log_print(to_string(Scanner)) },

	write_string("Input a sentence of the grammar: "),
	read_line_as_string(Result2),
	{ Result2 = ok(Input)
	; error("NO")
	}
	,
	{
	  TS = scan(Scanner,Input)
	;
	  error("lexical error")
	},
	write_string("Scanned\n"),
	print_all_tokens(TS,_),
	write_string("Scanned token sequence\n"),
	write_string(to_string(TS)),
	write_string("\nWith\n"),
	write(TS),
	nl.


:- pred (print_all_tokens(TS::in,TS::out,io__state::di,io__state::uo) is det)
	<= tokenSequence(TS).
print_all_tokens(TIn,TOut) -->
	{get_token(Tok,TIn,TMid)},
	( {Tok = end_of_file} ->
	    {Out = In,
	    TOut = TMid}
	;
	    print("\t"),
	    print(to_string(Tok)),
	    print("  =\t"),
	    print(Tok),
	    nl,
	    print_all_tokens(TMid,TOut)
	).

:- end_module scat.