/**
 * Provides abstract syntax tree representation.
 * @author Andre Platzer
 * @version 2003-08-18
 */
:- module syntaxTree.
:- interface.
:- import_module list, basecap, parsers.

/**
 * An abstract syntax tree with nodes marked with values of type T.
 * @author Andre Platzer
 * @version 2003-08-18
 */
:- type syntax_tree(T).
:- instance printable(syntax_tree(T)) <= printable(T).

/**
 * Creates an empty syntax tree.
 * @todo necessary?
 */
:- func init = syntax_tree(T).

/**
 * Make a new singular tree with a root marked with an element of T,
 * of no children.
 */
:- func node(T) = syntax_tree(T).

/**
 * Make a new tree with a root marked with an element of T,
 * and the specified list of children.
 */
:- func node(T, list(syntax_tree(T))) = syntax_tree(T).


/**
 * Make rule with default abstract syntax tree formation action.
 */
:- func rule(symbol, list(symbol)) = production is det.




:- implementation.

:- import_module int, string, list, require, std_util.
%% @see relation
%% @see graph

:- type syntax_tree(T)
	---> tree_node(
		  mark :: T,
		  children :: list(syntax_tree(T))
		 )
	;    empty_tree.

init = empty_tree.

node(M) = node(M, []).

node(M,L) = tree_node(M, L).

:- func print(int, syntax_tree(T)) = string <= printable(T).
print(Depth, tree_node(M, L)) = pad_left("", ' ', Depth)
        ++ "->" ++ to_string(M)
	++ "\n" ++ join_list("",map(print(Depth+2), L)).
print(_, empty_tree) = "".

:- instance printable(syntax_tree(T)) <= printable(T) where [
	to_string(Tree) = print(0, Tree)
 ].


rule(NT,RHS) = rule(NT,RHS,
		    (func(L)
		    = new_action_value(node(new_action_value(NT),
					    map(semidet_to_det((func(V::in) = (R::out) is semidet :-
							       (R2 = action_value_to_type(V) ->
								   R = R2
							       ;
								R = node(V)
							       )),
							       "cast action values to syntax tree"),
						L)
					   )))
		   ).


:- end_module syntaxTree.