/**
 * Synx, the syntactic parser generator.
 * @author Andre Platzer
 * @version 2003-08-19
 */
:- module synx.
:- interface.
:- import_module io.

:- pred main(io__state::di, io__state::uo) is det.

:- implementation.

:- import_module list, set, string, basecap, scanners, parsers, std_util, require,
	parsers__syntheticParser, parsers__sllParser,
	parsers__analyticParser, parsers__lr1Parser,
	parser_generator, parser_generator__lr1Generator, parser_generator__sllGenerator,
	grammar_analysis,
	syntaxTree,
	grammar_parser,
	scanner_generator, scanner_generator__lr0Generator,
	scanners__dfa.
:- import_module simple_tokenizer, token_impl, string_source.

main -->
	write_string("Welcome to Synx!\n"),
	io__command_line_arguments(Args),
	{ Args = [] ->
	    error("Pass grammar file at command line\nInvoke with synx <grammar.file>")
	;
	    GrammarFile = det_head(Args)
	},
	write_string("Grammar notation: "),
	open_input(GrammarFile, StreamResult),
	{ StreamResult = ok(GrammarStream)
	; error("Could not open file " ++ GrammarFile)
	},
	read_file_as_string(GrammarStream, Result),
	{ Result = ok(GNotation)
	; error("Could not read file " ++ GrammarFile)
	},
	close_input(GrammarStream),

	{
	 parse_grammar(GNotation, G, LexicalGrammar)
	;
	 error("Syntactically invalid grammar notation")
	},
	write_string("Grammar\n"),
	write_string(to_string(G)),
	nl,

	write_string("Generate Scanner\n"),
	{
	 (LexicalGrammar = scanner_generator__grammar([]) ->
	     error("No lexical grammar specified")
	     %Scanner = simple_tokenizer
	 ;   Scanner = dfa_scanner(LexicalGrammar)
	 ;
	     error("Lexical grammar cannot be parsed")
	 )
	},
	{ log_print(to_string(Scanner)) },
	write_string("Generate Parser\n"),
	{
	  Parser = sll_parser(G)
	;
	  error("Grammar cannot be parsed")
	},
	{ log_print(to_string(Parser)) },

	write_string("Input a sentence of the grammar: "),
	read_line_as_string(Result2),
	{ Result2 = ok(InputU),
	  %% @xxx this is cheating, but it avoids the need to introduce escape sequences in regular expression definitions.
	  Input = replace_all(replace_all(InputU, "\n", " "), "\r", " ")
	; error("NO")
	}
	,
	{
	  TS = scan(Scanner,string_tokenSequence(Input))
	;
	  error("lexical error")
	},
	{ log_print(to_string(TS)) },
	(
	  { Output = some_solution(parse(Parser, TS)),
	    log_print(to_string(Output))
	  }
	  %%, write(to_string(Output))
	;
	  { error("Sentence could not be parsed") }
	).

:- end_module synx.