/**
 * Provides concrete token representation.
 * @author Andre Platzer
 * @version 2003-08-17
 */
:- module token_impl.
:- interface.
:- import_module scanners, string, basecap.

:- type token_impl
	%% lexical tokens
	---> identifier(
		      signifier :: string
		     )
	;    operator(
		      string
		     )
	;    delimiter(
		       string
		      )
	;    string_literal(
		    string
		   )
	/**
	 * Explicit marker for end of token stream.
	 */
	;    endoffile.

:- instance printable(token_impl).
:- instance token(token_impl).

/**
 * Get token kind common for all identifier(A).
 * Convenience.
 */
:- func identifier = token_kind.
:- func string_literal = token_kind.


:- implementation.

:- import_module std_util, require.
:- import_module parsers, syntaxTree.

:- instance printable(token_impl) where [
        to_string(identifier(A)) = "\"" ++ A ++ "\"",
        to_string(operator(A)) = A,
        to_string(delimiter(A)) = A,
        to_string(string_literal(A)) = "\"" ++ A ++ "\"",
        to_string(endoffile) = "eof"
].

:- func interpret_value(token_impl) = string.
interpret_value(identifier(A)) = A.
interpret_value(operator(A)) = A.
interpret_value(delimiter(A)) = A.
interpret_value(string_literal(A)) = A.
interpret_value(endoffile) = A :- error("cannot get_value of " ++ to_string(endoffile)).

:- instance token(token_impl) where [
        func(get_value/1) is interpret_value,

        get_kind(identifier(A)) = make_token_kind("identifier" /*++ A*/),
        get_kind(operator(A)) = make_token_kind("op" ++ A),
        get_kind(delimiter(A)) = make_token_kind("delim" ++ A),
        get_kind(string_literal(A)) = make_token_kind("string_literal"),
        get_kind(endoffile) = make_token_kind("end_of_file"),

	end_of_file = endoffile
 ].


identifier = get_kind(identifier("any")).
string_literal = get_kind(string_literal("any")).

:- end_module token_impl.